defmodule Mash.Job do
  @moduledoc """
  Structure that represents the configuration and status of one Mash job.
  """

  defstruct name: nil, needs: [], run: nil, state: nil, task: nil, io_pid: nil

  def from_map(map) do
    struct(__MODULE__, map)
  end

  def validate(%__MODULE__{run: run, name: name} = job) do
    errors =
      [
        if(!is_function(run, 1), do: "run is required and must be a function with arity 1"),
        if(!is_atom(name), do: "name is required and must be an atom")
      ]
      |> Enum.reject(&is_nil(&1))

    if errors == [] do
      {:ok, job}
    else
      {:error, errors}
    end
  end
end
